@@ -7,16 +7,13 @@ import gzip
 import json
 from datetime import datetime
 import os
-import requests
 
 from ckanapi.errors import (NotFound, NotAuthorized, ValidationError,
     SearchIndexError)
 from ckanapi.cli import workers
 from ckanapi.cli.utils import completion_stats, compact_json, \
-    quiet_int_pipe, pretty_json
-
-DL_CHUNK_SIZE = 100 * 1024
-DATAPACKAGE_VERSION = '1.0-beta.10'
+    quiet_int_pipe
+from ckanapi.datapackage import create_datapackage, populate_datastore_fields
 
 
 def dump_things(ckan, thing, arguments,
@@ -31,7 +28,7 @@ def dump_things(ckan, thing, arguments,
     if worker_pool is None:
         worker_pool = workers.worker_pool
     if stdout is None:
-        stdout = getattr(sys.stdout, 'buffer', sys.stdout)
+        stdout = getattr(sys.__stdout__, 'buffer', sys.__stdout__)
     if stderr is None:
         stderr = getattr(sys.stderr, 'buffer', sys.stderr)
 
@@ -57,7 +54,10 @@ def dump_things(ckan, thing, arguments,
             'users': 'user_list',
             'related' :'related_list',
             }[thing]
-        names = ckan.call_action(get_thing_list, {})
+        params = dict(
+            all_fields=False,  # for user_list
+            )
+        names = ckan.call_action(get_thing_list, params)
 
     else:
         names = arguments['ID_OR_NAME']
@@ -65,6 +65,8 @@ def dump_things(ckan, thing, arguments,
     if names and isinstance(names[0], dict):
         names = [rec.get('name',rec.get('id')) for rec in names]
 
+    if arguments['--datapackages']:
+        arguments['--datastore-fields'] = True
     cmd = _worker_command_line(thing, arguments)
     processes = int(arguments['--processes'])
     if hasattr(ckan, 'parallel_limit'):
@@ -135,7 +137,7 @@ def dump_things_worker(ckan, thing, arguments,
         except IOError:
             pass
     if stdout is None:
-        stdout = getattr(sys.stdout, 'buffer', sys.stdout)
+        stdout = getattr(sys.__stdout__, 'buffer', sys.__stdout__)
         # hack so that "print debugging" can work in extension/ckan
         # code called by this worker
         sys.stdout = sys.stderr
@@ -170,66 +172,14 @@ def dump_things_worker(ckan, thing, arguments,
                 'include_datasets': False,
                 'include_password_hash': True,
                 })
-            reply(None, obj)
         except NotFound:
             reply('NotFound')
         except NotAuthorized:
             reply('NotAuthorized')
-
-
-def create_datapackage(record, base_path, stderr):
-    # TODO: how are we going to handle which resources to
-    # leave alone? They're very inconsistent in some instances
-    # And I can't imagine anyone wants to download a copy
-    # of, for example, the API base endpoint
-    resource_formats_to_ignore = ['API', 'api']
-    dataset_name = record.get('name', '') if record else ''
-
-    target_dir = '{base_path}/{name}/data'.format(
-        base_path=base_path,
-        name=dataset_name)
-
-    try:
-        os.makedirs(target_dir)
-    except Exception as e:
-        stderr.write(e.message)
-
-    for resource in record.get('resources', ''):
-        if resource.get('name') is not None:
-            resource_id = resource['name']
         else:
-            resource_id = resource['id']
-
-        resource_filename = os.path.split(resource['url'])[1]
-
-        output = os.path.join(target_dir, resource_filename)
-
-        # Resources can have a free-form address and no internal info, so in those cases
-        # we're going to merely save them using the UID. (If they even exist)
-        if output.endswith('/'):
-            output = os.path.join(output, resource_id)
-
-        resource['path'] = 'data' + output[len(target_dir):]
-
-        try:
-            if resource['format'] not in resource_formats_to_ignore:
-                r = requests.get(resource['url'], stream=True)
-                with open(output, 'wb') as f:
-                    for chunk in r.iter_content(chunk_size=DL_CHUNK_SIZE):
-                        if chunk: # filter out keep-alive new chunks
-                            f.write(chunk)
-                            f.flush()
-        except requests.ConnectionError:
-            stderr.write('URL {url} refused connection. The resource will not be downloaded\n'.format(url=resource['url']))
-        except requests.exceptions.RequestException as e:
-            stderr.write(e.message)
-            stderr.write('\n')
-
-    json_output_name = '{base_path}/{dataset_name}/datapackage.json'.format(
-        base_path=base_path, dataset_name=dataset_name)
-    with open(json_output_name, 'wb') as out:
-        out.write(pretty_json(dict(record, version=DATAPACKAGE_VERSION)))
-
+            if thing == 'datasets' and arguments['--datastore-fields']:
+                populate_datastore_fields(ckan, obj)
+            reply(None, obj)
 
 def _worker_command_line(thing, arguments):
     """
@@ -249,5 +199,6 @@ def _worker_command_line(thing, arguments):
         + a('--remote')
         + a('--apikey')
         + b('--get-request')
+        + b('--datastore-fields')
         + ['value-here-to-make-docopt-happy']
         )
